//
// Cleversafe open-source code header - Version 1.2 - February 15, 2008
//
// Cleversafe Dispersed Storage(TM) is software for secure, private and
// reliable storage of the world's data using information dispersal.
//
// Copyright (C) 2005-2008 Cleversafe, Inc.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
// USA.
//
// Contact Information: Cleversafe, 224 North Desplaines Street, Suite 500 
// Chicago IL 60661
// email licensing@cleversafe.org
//
// END-OF-HEADER
//-----------------------
// Author: Zach
//
// Date: Jun 15, 2007
//---------------------

package org.cleversafe.authentication;

import java.security.KeyPair;
import java.util.UUID;

import org.cleversafe.authentication.exceptions.CredentialsException;


/**
 * Interface for classes which load credentials.
 *
 */
public interface CredentialsManager
{
   /** 
    * 
    * @return The Credentials object for the specified account UUID.
    * @throws CredentialsException 
    */
   public Credentials getCredentials(UUID accountUUID) throws CredentialsException;
   
   /**
    * @return The grid account key pair for the specified account UUID.
    * @throws CredentialsException
    */
   public KeyPair getAccountKeyPair(UUID accountUUID) throws CredentialsException;
   
   /**
    * Performs a lookup or conversion from username to account identifier. An account
    * identifier is usually a UUID or GUID.
    * 
    * @param username The username to lookup.
    * @return The account identifier associated with the given username.
    * @throws CredentialsException
    */
   public UUID getAccountIdentifier( String username ) throws CredentialsException;
   
}


