//
// Cleversafe open-source code header - Version 1.2 - February 15, 2008
//
// Cleversafe Dispersed Storage(TM) is software for secure, private and
// reliable storage of the world's data using information dispersal.
//
// Copyright (C) 2005-2008 Cleversafe, Inc.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
// USA.
//
// Contact Information: Cleversafe, 224 North Desplaines Street, Suite 500 
// Chicago IL 60661
// email licensing@cleversafe.org
//
// END-OF-HEADER
//-----------------------
// Author: John Quigley <jquigley@cleversafe.com>
//
// Date: May 1, 2007
//---------------------

package org.cleversafe.codec;

import org.cleversafe.codec.exceptions.CodecEncodeException;
import org.cleversafe.codec.exceptions.CodecInvalidDataFormatException;
import org.cleversafe.codec.exceptions.CodecInvalidParametersException;
import org.cleversafe.codec.exceptions.CodecNotInitializedException;
import org.cleversafe.layer.grid.SourceName;

// TODO: Describe class or interface
public interface Encoder
{
   /**
    * 
    * @return if a encoder is ready for processing
    */
   boolean isInitialized();

   /**
    * Initializes encoder to begin a new encoding session
    * 
    */
   void reset(SourceName sourceName, long transactionId) throws CodecInvalidParametersException;

   /**
    * Process a new portion of data and returns something (may be empty)
    * 
    * @post isInitialized() == true
    * @param data
    *           to be processed
    * @return the result of decoding
    */
   byte[] process(byte[] data) throws CodecEncodeException, CodecInvalidDataFormatException,
         CodecNotInitializedException;

   /**
    * @post isInitialized() == false
    * @return bytes as the result of finalization
    */
   byte[] finish() throws CodecEncodeException, CodecInvalidDataFormatException,
         CodecNotInitializedException;

   byte[] finish(byte[] data) throws CodecEncodeException, CodecInvalidDataFormatException,
         CodecNotInitializedException;
}
